==> musashi-core-1.0.4-pre3.tar.gz: musashi-core-1.0.4-pre3/dm/pmmlagm/readdata2.c <==
//readdata.c 
//PMMLб 5/11
//ǡ򤪤ʤ
#include "product.c"
#include "encode2.c"
#include "decode2.c"

extern struct graph transaction[Transaction_num];
extern unsigned long transaction_num;
extern char encoding[40];
extern char copyright[256];
extern char modelname[256];

void toutf(char *filename); //ʸɤUTFݣѴؿ
int comp_func(const void *a,const void *b);

void readdata2(char* filename)
{
  unsigned long i,j,k,traid=0,size,idnum=0,nodenum1,nodenum2,nodetypemax=0,nodetypenum,linktypemax=0,linktypenum,dummy2[100];
  char fname[300],ch,idname[Transaction_num][15],nodetype[15],nodetypenames[100][20],linktype[15],linktypenames[100][15];
  cmatrix tramat,adjmat[Transaction_num];
  
  xmlDocPtr doc; //xmlfile ؤΥݥ
  xmlNodePtr cur; //Node ؤΥݥ
  char g_id[10]; //IDݻΤѿ
  char v_id[10];
  char transacfile[300];
  FILE *fp;

  
  xmlNodePtr curtmp,curtmp2;
  int flg = 1; //GraphData ǤĤäå뤿Υե饰
  int chkflg = 0; //graphtype ΥåλĴ٤뤿Υե饰
  
  g_type = 2;


  //libxmlĤʸɤUTFݣѴ
  /* xmlParseFilexmlReadFileѹ뤳ȤtoutfѤʤ by Ham*/
  /* toutf(filename); */

  /* sprintf(fname,"%s/xt##%d-utf",wfp,(int)getpid());*/

  /*doc = xmlParseFile(fname); //pmmlfile ѡΤΥݥ*/
  /*gmlFile ѡΤΥݥ by Ham*/
  doc = xmlReadFile(filename,NULL,0);
  
  //ѡ˼Ԥ票顼ɽƽλ
  if(doc == NULL){
    mssShowErrMsg("Document not parsed successfully");
    mssEnd(mssErrorNoDefault);
  }

  /* doc->encoding Ѥencoding˥ԡ by Ham*/
  strcpy(encoding,doc->encoding);

  cur = xmlDocGetRootElement(doc); //롼ȥΡɤؤΥݥ󥿤
  
  //ΥɥȤä票顼ɽƽλ
  if (cur == NULL) {
    mssShowErrMsg("empty document");
    xmlFreeDoc(doc);
    mssEnd(mssErrorNoDefault);
  }

  cur = cur -> xmlChildrenNode;
  
  //GraphDataǤõڤ̵ͭդΥå
  //copyrightμ
  
  while (cur != NULL) {

    if(!(xmlStrcmp(cur -> name,(const xmlChar *)"Header"))){

      strcpy(copyright,xmlGetProp(cur,"copyright"));

    }
    if (!(xmlStrcmp(cur -> name, (const xmlChar *)"GraphData"))){
      
      
      curtmp = cur -> xmlChildrenNode;
      
      while(curtmp != NULL && !chkflg){

        if(!(xmlStrcmp(curtmp -> name,(const xmlChar *)"Graph"))){

          curtmp2 = curtmp -> xmlChildrenNode;

          while(curtmp2 != NULL && !chkflg){

            if(!(xmlStrcmp(curtmp2 -> name,(const xmlChar *) "Edge"))){

              if(strcmp(xmlGetProp(curtmp2,"edgeType"),"directed")==0){

                g_type = 1;
                break;
              }
            }
            curtmp2 = curtmp2 -> next;

          }

        }

        curtmp = curtmp -> next;

      }

      curtmp = cur -> xmlChildrenNode;
      flg = 0;

      while(curtmp != NULL){
  
        if(!(xmlStrcmp(curtmp -> name, (const xmlChar *)"Graph"))){

	  mssGV.inCnt++;

          size = 0;
          strcpy(g_id,xmlGetProp(curtmp, "graphId"));
          curtmp2 = curtmp -> xmlChildrenNode;

	  //եĴ٤

          while(curtmp2 != NULL){
            if(!(xmlStrcmp(curtmp2 -> name, (const xmlChar *)"Vertex"))){
              size++;
            }
            curtmp2 = curtmp2 -> next;
          }

	  //եIDǼ

          strcpy(idname[traid],g_id);
          transaction[traid].size=size;

	  /* γݡ */
          adjmat[traid]=cmat(size,size);

          for(i=0;i<size;i++){
            for(j=0;j<size;j++){
              adjmat[traid][i][j]=0;
            }
          }

	  transaction[traid].nodetypes = (unsigned long*)malloc(sizeof(unsigned long)*size);    
          traid++;
    
          curtmp2 = curtmp -> xmlChildrenNode;    
    
	  //VertexǤ饰ID٥ʤɤФǼ
          while(curtmp2 != NULL){
      
            if(!(xmlStrcmp(curtmp2 -> name, (const xmlChar *) "Vertex"))){

              
	      if(xmlGetProp(curtmp2 -> xmlChildrenNode -> next,"value") != NULL){
		strcpy(nodetype,xmlGetProp(curtmp2 -> xmlChildrenNode -> next,"value"));
	      }
	      else{
		strcpy(nodetype,"NodeLabelNA");
	      }
      
	      strcpy(v_id,xmlGetProp(curtmp2,"vertexId"));
              nodenum1 = atoi(v_id);
	      ch = 0;
        
              for(nodetypenum=0;nodetypenum<nodetypemax;nodetypenum++){
                if(strcmp(nodetypenames[nodetypenum],nodetype)==0){
                  nodetypenum++;
                  ch=1;
                  break;
                }
              }

              if(ch==0){
                strcpy(nodetypenames[nodetypemax],nodetype);
                nodetypemax++;
                nodetypenum=nodetypemax;
              }
	    
	      for(idnum = 0;idnum < traid;idnum++){
		if(strcmp(idname[idnum],g_id)==0)
		  break;
	      }

              transaction[idnum].nodetypes[nodenum1-1]=nodetypenum;

            }
      
            curtmp2 = curtmp2 -> next;
      
          }
    
          curtmp2 = curtmp -> xmlChildrenNode;

          //EdgeǤ饰ID٥ʤɤФǼ
          while(curtmp2 != NULL){
      
            if(!(xmlStrcmp(curtmp2 -> name,(const xmlChar *) "Edge"))){
     
              nodenum1 = atoi(xmlGetProp(curtmp2,"bgnVertexId"));
              nodenum2 = atoi(xmlGetProp(curtmp2,"endVertexId"));
	    
              if(xmlGetProp(curtmp2 -> xmlChildrenNode -> next,"value") != NULL){
		strcpy(linktype,xmlGetProp(curtmp2 -> xmlChildrenNode -> next,"value"));
	      }
	      else{
		strcpy(linktype,"LinkLabelNA");
	      }
              ch=0;
              for(linktypenum=0;linktypenum<linktypemax;linktypenum++){
                if(strcmp(linktypenames[linktypenum],linktype)==0){
                  linktypenum++;
                  ch=1;
                  break;
                }
              }
              if(ch==0){
                strcpy(linktypenames[linktypemax],linktype);
                linktypemax++;
                linktypenum=linktypemax;
              }
	     
              if(g_type==1){
                adjmat[idnum][nodenum1-1][nodenum2-1]=(char)linktypenum;
              }
              else{
                adjmat[idnum][nodenum1-1][nodenum2-1]=(char)linktypenum;
                adjmat[idnum][nodenum2-1][nodenum1-1]=(char)linktypenum;
              }

            }

            curtmp2 = curtmp2 -> next;

          }

        }
        curtmp = curtmp -> next;
      
      }

    }

    cur = cur -> next;
  
  }
  
      
  //GraphData ǤĤʤä票顼ɽƽλ
  if(flg){
    mssShowErrMsg("not found GraphData element");
    xmlFreeDoc(doc);
    mssEnd(mssErrorNoDefault);
  }
  
  xmlFreeDoc(doc);
 
  //ǡե˽
  sprintf(transacfile,"%s/xt##%d-transaction",wfp,(int)getpid());
  fp=fopen(transacfile,"w"); 
  
  transaction_num=traid;
  ltypenum=(char)linktypemax;
  tramat=cmat(150,150);
  for(i=0;i<transaction_num;i++){
    for(j=0;j<transaction[i].size;j++)
      dummy2[j]=transaction[i].nodetypes[j];
    qsort(transaction[i].nodetypes,transaction[i].size,sizeof(unsigned long),comp_func);
    for(j=0;j<transaction[i].size;j++){
      for(k=0;k<transaction[i].size;k++){
  tramat[j][k]=0;
      }
    }
    for(j=0;j<transaction[i].size;j++){
      for(k=0;k<transaction[i].size;k++){
  if(dummy2[k]!=0){
    if(transaction[i].nodetypes[j]==dummy2[k]){
      tramat[k][j]=1;
      dummy2[k]=0;
      k=transaction[i].size-1;
    }
  }
      }
    }
    /* ܹѤ */
    product2(adjmat[i],adjmat[i],tramat,transaction[i].size);
    transaction[i].code=(char *)malloc(sizeof(char)*transaction[i].size*(transaction[i].size-1)/2);
    fdispcmat(adjmat[i],transaction[i].size,transaction[i].size,fp); /* transaction.txt ˹¸ */
    /* ܹ򥳡ɲ */
    encode2(transaction[i].code,adjmat[i],transaction[i].size);
  }
  fclose(fp);
  

  //ΡɤΥ٥ꥹȤե˽
  sprintf(fname,"%s/xt##%d-nwrd",wfp,(int)getpid());
  fp=fopen(fname,"w");
    
  for(i=0;i<nodetypemax;i++){
    fprintf(fp,"%s\n",nodetypenames[i]);
  }

  fclose(fp);

  //󥯤Υ٥ꥹȤե˽
  sprintf(fname,"%s/xt##%d-lwrd",wfp,(int)getpid());
  fp=fopen(fname,"w");
  for(i=0;i<linktypemax;i++){
    fprintf(fp,"%s\n",linktypenames[i]);
  }
  fclose(fp);
  ntypenum=nodetypemax;
  ltypenum=(char)linktypemax;


}

void toutf(char *filename)
{
  char buf[256],bgntag[20],key[20],enci[256],enco[512],fname[300];
  char *tmp1=NULL;
  char *tmp2=NULL;
  int flg = 0;
  size_t encilen,encolen;

#ifdef MssNeedLibiconv
  const char *encip;     /*iconv ʸ򼨤ݥ */
#else
  char *encip;
#endif
  char *encop;
  
  iconv_t *icid;

  FILE *fp1;
  FILE *fp2;

  strcpy(bgntag,"<?xml");
  strcpy(key,"encoding");

  fp1=fopen(filename,"r");
  
  if(fp1==NULL){  

      mssShowErrMsg("եΥץ˼Ԥޤ");
      mssEnd(mssErrorNoDefault);
  }

  while(fgets(buf,256,fp1)){
    
    if(strstr(buf,bgntag)){
      
      if((tmp1=strstr(buf,key))){
  tmp2 = tmp1;
  while(tmp2 != NULL){
    
    if(*tmp2 == '"'){
      if(flg==0){
        tmp1 = tmp2 + 1;
      }
      else if(flg ==1){
        break;
      }
      flg++;
    }
    
    tmp2++;
  }
      }
      
    }
    if(flg==1){
      break;
    }
  }

  strncpy(encoding,tmp1,tmp2-tmp1);
  encoding[tmp2-tmp1] = '\0';

  icid=iconv_open("UTF-8",encoding);
  
  rewind(fp1);
  sprintf(fname,"%s/xt##%d-utf",wfp,(int)getpid());

  fp2=fopen(fname,"w");

  
  while(fgets(enci,256,fp1)!=NULL){
    
    encip = enci;
    encop = enco;
    
    encilen = strlen(enci)+1;
    encolen = encilen * 2;

    if(-1==iconv(icid,&encip,&encilen,&encop,&encolen)){
      mssShowErrMsg("ʸѴ˼Ԥޤ");
      mssEnd(mssErrorNoDefault);
    }
    
    fprintf(fp2,"%s",enco);
  }

  fclose(fp1);
  fclose(fp2);
}     

int comp_func(const void *a,const void *b)
{
  return *(int *)a-*(int *)b;
}

==> musashi-core-2.0.1.tar.gz: musashi-core-2.0.1/dm/pmmlagm/readdata2.c <==
//readdata.c を改変
//PMML対応 5/11光永
//データの前処理をおこなう
#include "product.c"
#include "encode2.c"
#include "decode2.c"

extern struct graph transaction[Transaction_num];
extern unsigned long transaction_num;
extern char encoding[40];
extern char copyright[256];
extern char modelname[256];

void toutf(char *filename); //文字コードをUTF−８に変換する関数
int comp_func(const void *a,const void *b);

void readdata2(char* filename)
{
  unsigned long i,j,k,traid=0,size,idnum=0,nodenum1,nodenum2,nodetypemax=0,nodetypenum,linktypemax=0,linktypenum,dummy2[100];
  char fname[300],ch,idname[Transaction_num][15],nodetype[15],nodetypenames[100][20],linktype[15],linktypenames[100][15];
  cmatrix tramat,adjmat[Transaction_num];
  
  xmlDocPtr doc; //xmlfile へのポインタ
  xmlNodePtr cur; //Node へのポインタ
  char g_id[10]; //グラフID保持のための変数
  char v_id[10];
  char transacfile[300];
  FILE *fp;

  
  xmlNodePtr curtmp,curtmp2;
  int flg = 1; //GraphData 要素が見つかったチェックするためのフラグ
  int chkflg = 0; //graphtype のチェックが完了したか調べるためのフラグ
  
  g_type = 2;


  //libxmlをつかうために文字コードをUTF−８に変換
  /* xmlParseFileをxmlReadFileに変更することでtoutfを利用しない by Ham*/
  /* toutf(filename); */

  /* sprintf(fname,"%s/xt##%d-utf",wfp,(int)getpid());*/

  /*doc = xmlParseFile(fname); //pmmlfile パースのためのポインタ*/
  /*gmlFile パースのためのポインタ by Ham*/
  doc = xmlReadFile(filename,NULL,0);
  
  //パースに失敗した場合エラーを表示して終了
  if(doc == NULL){
    mssShowErrMsg("Document not parsed successfully");
    mssEnd(mssErrorNoDefault);
  }

  /* doc->encoding を出力用のencodingにコピー by Ham*/
  strcpy(encoding,doc->encoding);

  cur = xmlDocGetRootElement(doc); //ルートノードへのポインタを獲得
  
  //空のドキュメントだった場合エラーを表示して終了
  if (cur == NULL) {
    mssShowErrMsg("empty document");
    xmlFreeDoc(doc);
    mssEnd(mssErrorNoDefault);
  }

  cur = cur -> xmlChildrenNode;
  
  //GraphData要素の探索及び有向、無向グラフのチェック
  //copyrightの取得
  
  while (cur != NULL) {

    if(!(xmlStrcmp(cur -> name,(const xmlChar *)"Header"))){

      strcpy(copyright,xmlGetProp(cur,"copyright"));

    }
    if (!(xmlStrcmp(cur -> name, (const xmlChar *)"GraphData"))){
      
      
      curtmp = cur -> xmlChildrenNode;
      
      while(curtmp != NULL && !chkflg){

        if(!(xmlStrcmp(curtmp -> name,(const xmlChar *)"Graph"))){

          curtmp2 = curtmp -> xmlChildrenNode;

          while(curtmp2 != NULL && !chkflg){

            if(!(xmlStrcmp(curtmp2 -> name,(const xmlChar *) "Edge"))){

              if(strcmp(xmlGetProp(curtmp2,"edgeType"),"directed")==0){

                g_type = 1;
                break;
              }
            }
            curtmp2 = curtmp2 -> next;

          }

        }

        curtmp = curtmp -> next;

      }

      curtmp = cur -> xmlChildrenNode;
      flg = 0;

      while(curtmp != NULL){
  
        if(!(xmlStrcmp(curtmp -> name, (const xmlChar *)"Graph"))){

	  mssGV.inCnt++;

          size = 0;
          strcpy(g_id,xmlGetProp(curtmp, "graphId"));
          curtmp2 = curtmp -> xmlChildrenNode;

	  //グラフサイズを調べる

          while(curtmp2 != NULL){
            if(!(xmlStrcmp(curtmp2 -> name, (const xmlChar *)"Vertex"))){
              size++;
            }
            curtmp2 = curtmp2 -> next;
          }

	  //グラフサイズとIDを格納

          strcpy(idname[traid],g_id);
          transaction[traid].size=size;

	  /* 行列の確保＆初期化 */
          adjmat[traid]=cmat(size,size);

          for(i=0;i<size;i++){
            for(j=0;j<size;j++){
              adjmat[traid][i][j]=0;
            }
          }

	  transaction[traid].nodetypes = (unsigned long*)malloc(sizeof(unsigned long)*size);    
          traid++;
    
          curtmp2 = curtmp -> xmlChildrenNode;    
    
	  //Vertex要素からグラフID、ラベルなどを取り出し格納
          while(curtmp2 != NULL){
      
            if(!(xmlStrcmp(curtmp2 -> name, (const xmlChar *) "Vertex"))){

              
	      if(xmlGetProp(curtmp2 -> xmlChildrenNode -> next,"value") != NULL){
		strcpy(nodetype,xmlGetProp(curtmp2 -> xmlChildrenNode -> next,"value"));
	      }
	      else{
		strcpy(nodetype,"NodeLabelNA");
	      }
      
	      strcpy(v_id,xmlGetProp(curtmp2,"vertexId"));
              nodenum1 = atoi(v_id);
	      ch = 0;
        
              for(nodetypenum=0;nodetypenum<nodetypemax;nodetypenum++){
                if(strcmp(nodetypenames[nodetypenum],nodetype)==0){
                  nodetypenum++;
                  ch=1;
                  break;
                }
              }

              if(ch==0){
                strcpy(nodetypenames[nodetypemax],nodetype);
                nodetypemax++;
                nodetypenum=nodetypemax;
              }
	    
	      for(idnum = 0;idnum < traid;idnum++){
		if(strcmp(idname[idnum],g_id)==0)
		  break;
	      }

              transaction[idnum].nodetypes[nodenum1-1]=nodetypenum;

            }
      
            curtmp2 = curtmp2 -> next;
      
          }
    
          curtmp2 = curtmp -> xmlChildrenNode;

          //Edge要素からグラフID、ラベルなどを取り出し格納
          while(curtmp2 != NULL){
      
            if(!(xmlStrcmp(curtmp2 -> name,(const xmlChar *) "Edge"))){
     
              nodenum1 = atoi(xmlGetProp(curtmp2,"bgnVertexId"));
              nodenum2 = atoi(xmlGetProp(curtmp2,"endVertexId"));
	    
              if(xmlGetProp(curtmp2 -> xmlChildrenNode -> next,"value") != NULL){
		strcpy(linktype,xmlGetProp(curtmp2 -> xmlChildrenNode -> next,"value"));
	      }
	      else{
		strcpy(linktype,"LinkLabelNA");
	      }
              ch=0;
              for(linktypenum=0;linktypenum<linktypemax;linktypenum++){
                if(strcmp(linktypenames[linktypenum],linktype)==0){
                  linktypenum++;
                  ch=1;
                  break;
                }
              }
              if(ch==0){
                strcpy(linktypenames[linktypemax],linktype);
                linktypemax++;
                linktypenum=linktypemax;
              }
	     
              if(g_type==1){
                adjmat[idnum][nodenum1-1][nodenum2-1]=(char)linktypenum;
              }
              else{
                adjmat[idnum][nodenum1-1][nodenum2-1]=(char)linktypenum;
                adjmat[idnum][nodenum2-1][nodenum1-1]=(char)linktypenum;
              }

            }

            curtmp2 = curtmp2 -> next;

          }

        }
        curtmp = curtmp -> next;
      
      }

    }

    cur = cur -> next;
  
  }
  
      
  //GraphData 要素が見つからなかった場合エラーを表示して終了
  if(flg){
    mssShowErrMsg("not found GraphData element");
    xmlFreeDoc(doc);
    mssEnd(mssErrorNoDefault);
  }
  
  xmlFreeDoc(doc);
 
  //前処理したデータをファイルに出力
  sprintf(transacfile,"%s/xt##%d-transaction",wfp,(int)getpid());
  fp=fopen(transacfile,"w"); 
  
  transaction_num=traid;
  ltypenum=(char)linktypemax;
  tramat=cmat(150,150);
  for(i=0;i<transaction_num;i++){
    for(j=0;j<transaction[i].size;j++)
      dummy2[j]=transaction[i].nodetypes[j];
    qsort(transaction[i].nodetypes,transaction[i].size,sizeof(unsigned long),comp_func);
    for(j=0;j<transaction[i].size;j++){
      for(k=0;k<transaction[i].size;k++){
  tramat[j][k]=0;
      }
    }
    for(j=0;j<transaction[i].size;j++){
      for(k=0;k<transaction[i].size;k++){
  if(dummy2[k]!=0){
    if(transaction[i].nodetypes[j]==dummy2[k]){
      tramat[k][j]=1;
      dummy2[k]=0;
      k=transaction[i].size-1;
    }
  }
      }
    }
    /* 隣接行列を正規形に変える */
    product2(adjmat[i],adjmat[i],tramat,transaction[i].size);
    transaction[i].code=(char *)malloc(sizeof(char)*transaction[i].size*(transaction[i].size-1)/2);
    fdispcmat(adjmat[i],transaction[i].size,transaction[i].size,fp); /* transaction.txt に行列を保存 */
    /* 隣接行列をコード化 */
    encode2(transaction[i].code,adjmat[i],transaction[i].size);
  }
  fclose(fp);
  

  //ノードのラベルリストをファイルに出力
  sprintf(fname,"%s/xt##%d-nwrd",wfp,(int)getpid());
  fp=fopen(fname,"w");
    
  for(i=0;i<nodetypemax;i++){
    fprintf(fp,"%s\n",nodetypenames[i]);
  }

  fclose(fp);

  //リンクのラベルリストをファイルに出力
  sprintf(fname,"%s/xt##%d-lwrd",wfp,(int)getpid());
  fp=fopen(fname,"w");
  for(i=0;i<linktypemax;i++){
    fprintf(fp,"%s\n",linktypenames[i]);
  }
  fclose(fp);
  ntypenum=nodetypemax;
  ltypenum=(char)linktypemax;


}

void toutf(char *filename)
{
  char buf[256],bgntag[20],key[20],enci[256],enco[512],fname[300];
  char *tmp1=NULL;
  char *tmp2=NULL;
  int flg = 0;
  size_t encilen,encolen;

#ifdef MssNeedLibiconv
  const char *encip;     /*iconv用 入力文字列を示すポインタ */
#else
  char *encip;
#endif
  char *encop;
  
  iconv_t *icid;

  FILE *fp1;
  FILE *fp2;

  strcpy(bgntag,"<?xml");
  strcpy(key,"encoding");

  fp1=fopen(filename,"r");
  
  if(fp1==NULL){  

      mssShowErrMsg("ファイルのオープンに失敗しました");
      mssEnd(mssErrorNoDefault);
  }

  while(fgets(buf,256,fp1)){
    
    if(strstr(buf,bgntag)){
      
      if((tmp1=strstr(buf,key))){
  tmp2 = tmp1;
  while(tmp2 != NULL){
    
    if(*tmp2 == '"'){
      if(flg==0){
        tmp1 = tmp2 + 1;
      }
      else if(flg ==1){
        break;
      }
      flg++;
    }
    
    tmp2++;
  }
      }
      
    }
    if(flg==1){
      break;
    }
  }

  strncpy(encoding,tmp1,tmp2-tmp1);
  encoding[tmp2-tmp1] = '\0';

  icid=iconv_open("UTF-8",encoding);
  
  rewind(fp1);
  sprintf(fname,"%s/xt##%d-utf",wfp,(int)getpid());

  fp2=fopen(fname,"w");

  
  while(fgets(enci,256,fp1)!=NULL){
    
    encip = enci;
    encop = enco;
    
    encilen = strlen(enci)+1;
    encolen = encilen * 2;

    if(-1==iconv(icid,&encip,&encilen,&encop,&encolen)){
      mssShowErrMsg("文字コード変換に失敗しました");
      mssEnd(mssErrorNoDefault);
    }
    
    fprintf(fp2,"%s",enco);
  }

  fclose(fp1);
  fclose(fp2);
}     

int comp_func(const void *a,const void *b)
{
  return *(int *)a-*(int *)b;
}
